# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 9, 2011

@author: evan
'''
from kayako.tests import KayakoAPITest
class TestStaff(KayakoAPITest):

    def test_add_get_nonexistant(self):
        from kayako.objects import Staff
        obj = self.api.get(Staff, '123123')
        assert obj is None

    def test_add_get_bare(self):
        from kayako.objects import Staff
        obj = self.api.create(Staff, firstname='DELETEME', lastname='DELETEME', username='DELETEME', email='deleteme@example.com', password='DELETEME', staffgroupid=1)
        obj.add()
        obj2 = self.api.get(Staff, obj.id) # Shouldn't raise errors
        obj.delete()
        assert obj2 is not None

    def test_add_get_full(self):
        from kayako.objects import Staff
        obj = self.api.create(Staff, firstname='DELETEME', lastname='DELETEME', username='DELETEME', email='deleteme@example.com', password='DELETEME', staffgroupid=1,
                              designation='TEST', mobilenumber='123-456-7890', signature='TEST', isenabled=False, greeting='Mr.', timezone='MST', enabledst=True)
        obj.add()
        obj2 = self.api.get(Staff, obj.id) # Shouldn't raise errors
        obj.delete()
        assert obj2 is not None

    def test_get_staff(self):
        from kayako.objects import Staff
        d = self.api.get(Staff, 1)
        assert 'Staff ' in str(d)
        self.assertEqual(d.id, 1)

    def test_add_staff_missing_firstname(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Staff
        staff = self.api.create(Staff)

        staff.lastname = 'test_lastname'
        staff.username = 'test_username'
        staff.password = 'test_password'
        staff.staffgroupid = 1
        staff.email = 'test@example.com'

        self.assertRaises(KayakoRequestError, staff.add)

    def test_add_staff_missing_lastname(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Staff
        staff = self.api.create(Staff)

        staff.firstname = 'test_firstname'
        staff.username = 'test_username'
        staff.password = 'test_password'
        staff.staffgroupid = 1
        staff.email = 'test@example.com'

        self.assertRaises(KayakoRequestError, staff.add)

    def test_add_staff_missing_username(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Staff
        staff = self.api.create(Staff)

        staff.firstname = 'test_firstname'
        staff.lastname = 'test_lastname'
        staff.password = 'test_password'
        staff.staffgroupid = 1
        staff.email = 'test@example.com'

        self.assertRaises(KayakoRequestError, staff.add)

    def test_add_staff_missing_password(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Staff
        staff = self.api.create(Staff)

        staff.firstname = 'test_firstname'
        staff.lastname = 'test_lastname'
        staff.username = 'test_username'
        staff.staffgroupid = 1
        staff.email = 'test@example.com'

        self.assertRaises(KayakoRequestError, staff.add)

    def test_add_staff_missing_staffgroupid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Staff
        staff = self.api.create(Staff)

        staff.firstname = 'test_firstname'
        staff.lastname = 'test_lastname'
        staff.username = 'test_username'
        staff.password = 'test_password'
        staff.email = 'test@example.com'

        self.assertRaises(KayakoRequestError, staff.add)

    def test_add_staff_missing_email(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Staff
        staff = self.api.create(Staff)

        staff.firstname = 'test_firstname'
        staff.lastname = 'test_lastname'
        staff.username = 'test_username'
        staff.password = 'test_password'
        staff.staffgroupid = 1

        self.assertRaises(KayakoRequestError, staff.add)


    def test_get_all(self):
        from kayako.objects import Staff
        result = self.api.get_all(Staff)
        assert len(result)

    def test_add_save_delete(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import Staff

        staff = self.api.create(Staff)
        staff.firstname = 'DELETE_ME'
        staff.lastname = 'test_lastname'
        staff.username = 'test_username'
        staff.password = 'test_password'
        staff.staffgroupid = 1
        staff.email = 'test@example.com'
        staff.add()
        assert staff.id is not UnsetParameter
        staff.firstname = 'DELETE_ME2'
        staff.save()
        staff.delete()

        found_error = False
        all_staff = self.api.get_all(Staff)
        for staff in all_staff:
            if staff.firstname == 'DELETE_ME' or staff.firstname == 'DELETE_ME2':
                staff.delete()
                found_error = True
        if found_error:
            assert False, 'Found an error, Staff did not delete correctly.'

    def test_delete_unadded(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Staff
        staff = self.api.create(Staff)
        self.assertRaises(KayakoRequestError, staff.delete)

class TestStaffGroup(KayakoAPITest):

    def test_add_get_nonexistant(self):
        from kayako.objects import StaffGroup
        obj = self.api.get(StaffGroup, 123123123)
        assert obj is None

    def test_add_get_bare(self):
        from kayako.objects import StaffGroup
        obj = self.api.create(StaffGroup, title='DELETEME', isadmin=False)
        obj.add()
        obj2 = self.api.get(StaffGroup, obj.id) # Shouldn't raise errors
        obj.delete()
        assert obj2 is not None

    def test_add_get_full(self):
        from kayako.objects import StaffGroup
        obj = self.api.create(StaffGroup, title='DELETEME', isadmin=False)
        obj.add()
        obj2 = self.api.get(StaffGroup, obj.id) # Shouldn't raise errors
        obj.delete()
        assert obj2 is not None

    def test_get_staffgroup(self):
        from kayako.objects import StaffGroup
        d = self.api.get(StaffGroup, 1)
        self.assertEqual(d.id, 1)

    def test_add_staff_missing_title(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import StaffGroup
        staffgroup = self.api.create(StaffGroup)

        staffgroup.isadmin = 0

        self.assertRaises(KayakoRequestError, staffgroup.add)

    def test_add_staff_missing_isadmin(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import StaffGroup
        staffgroup = self.api.create(StaffGroup)

        staffgroup.title = 'test_title'

        self.assertRaises(KayakoRequestError, staffgroup.add)

    def test_get_all(self):
        from kayako.objects import StaffGroup
        result = self.api.get_all(StaffGroup)
        assert len(result)

    def test_add_save_delete(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import StaffGroup

        staffgroup = self.api.create(StaffGroup)
        staffgroup.title = 'DELETE_ME'
        staffgroup.isadmin = 0
        staffgroup.add()
        assert staffgroup.id is not UnsetParameter
        staffgroup.title = 'DELETE_ME2'
        staffgroup.save()
        staffgroup.delete()

        found_error = False
        all_staff_groups = self.api.get_all(StaffGroup)
        for staffgroup in all_staff_groups:
            if staffgroup.title == 'DELETE_ME' or staffgroup.title == 'DELETE_ME2':
                staffgroup.delete()
                found_error = True
        if found_error:
            assert False, 'Found an error, StaffGroup did not delete correctly.'

    def test_delete_unadded(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import StaffGroup
        staffgroup = self.api.create(StaffGroup)
        self.assertRaises(KayakoRequestError, staffgroup.delete)




# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 5, 2011

@author: evan
'''

from kayako.tests import KayakoAPITest

class TestKayakoAPI(KayakoAPITest):

    def test_init_without_url(self):
        from kayako.api import KayakoAPI
        from kayako.exception import KayakoInitializationError
        self.assertRaises(KayakoInitializationError, KayakoAPI, None, 'key', 'secret')

    def test_init_without_key(self):
        from kayako.api import KayakoAPI
        from kayako.exception import KayakoInitializationError
        self.assertRaises(KayakoInitializationError, KayakoAPI, 'url', None, 'secret')

    def test_init_without_secret(self):
        from kayako.api import KayakoAPI
        from kayako.exception import KayakoInitializationError
        self.assertRaises(KayakoInitializationError, KayakoAPI, 'url', 'key', None)

    def test__sanitize_paramter_list(self):
        api = self.api
        self.assertEqual(api._sanitize_parameter(['a', 'b', '', None, 'c']), ['a', 'b', 'c'])

    def test__sanitize_paramter_number(self):
        api = self.api
        self.assertEqual(api._sanitize_parameter(123), '123')

    def test__sanitize_paramter_none(self):
        api = self.api
        self.assertEqual(api._sanitize_parameter(None), '')

    def test__post_data_none(self):
        api = self.api
        sanitized = api._sanitize_parameters(data=None)
        results = api._post_data(**sanitized)
        self.assertEqual(results, 'data=')

    def test__post_data_array(self):
        api = self.api
        sanitized = api._sanitize_parameters(data=['abc', '', None, '123'])
        results = api._post_data(**sanitized)
        self.assertEqual(results, 'data[]=abc&data[]=123')

    def test__post_data_empty_array(self):
        api = self.api
        sanitized = api._sanitize_parameters(data=['', None])
        results = api._post_data(**sanitized)
        self.assertEqual(results, 'data[]=')

    def test__post_data_date(self):
        import time
        from datetime import datetime
        api = self.api

        date = datetime(2011, 5, 11, 12, 42, 46, 977079)
        timestamp = int(time.mktime(date.timetuple()))

        sanitized = api._sanitize_parameters(date=date)
        results = api._post_data(**sanitized)
        self.assertEqual(results, 'date=%s' % timestamp)

    def test__post_data_FOREVER(self):
        from kayako.core.lib import FOREVER
        api = self.api
        sanitized = api._sanitize_parameters(date=FOREVER)
        results = api._post_data(**sanitized)
        self.assertEqual(results, 'date=0')

    def test__post_data_int(self):
        api = self.api
        sanitized = api._sanitize_parameters(data=123)
        results = api._post_data(**sanitized)
        self.assertEqual(results, 'data=123')

    def test__post_data_str(self):
        api = self.api
        sanitized = api._sanitize_parameters(data='abc')
        results = api._post_data(**sanitized)
        self.assertEqual(results, 'data=abc')

    def test__post_data_true(self):
        api = self.api
        sanitized = api._sanitize_parameters(data=True)
        results = api._post_data(**sanitized)
        self.assertEqual(results, 'data=1')

    def test__post_data_false(self):
        api = self.api
        sanitized = api._sanitize_parameters(data=False)
        results = api._post_data(**sanitized)
        self.assertEqual(results, 'data=0')

    def test_signature(self):
        ''' Test the signature generation process '''
        import hmac
        import base64
        import urllib
        import hashlib

        secretkey = "secretkey"
        # Generates a random string of ten digits
        salt = '1234567890'
        # Computes the signature by hashing the salt with the secret key as the key
        signature = hmac.new(secretkey, msg=salt, digestmod=hashlib.sha256).digest()
        # base64 encode...
        encoded_signature = base64.b64encode(signature)
        # urlencode...
        url_encoded_signature = urllib.quote(encoded_signature)
        assert url_encoded_signature == 'VKjt8M54liY6xq1UuhUYH5BFp1RUqHekqytgLPrVEA0%3D'

    def test_get(self):
        r = self.api._request('/Core/Test', 'GET')
        assert r.read()
        assert r.getcode() == 200, r.getcode()
        r = self.api._request('/Core/Test', 'GET', test='just a test')
        assert r.read()
        assert r.getcode() == 200, r.getcode()
        r = self.api._request('/Core/Test/1', 'GET')
        assert r.read()
        assert r.getcode() == 200, r.getcode()
        r = self.api._request('/Core/Test/1', 'GET', test='just a test')
        assert r.read()
        assert r.getcode() == 200, r.getcode()

    def test_post(self):
        r = self.api._request('/Core/Test', 'POST')
        assert r.read()
        assert r.getcode() == 200, r.getcode()

    def test_put(self):
        r = self.api._request('/Core/Test/1', 'PUT', x=234)
        assert r.read()
        assert r.getcode() == 200, r.getcode()

    def test_delete(self):
        r = self.api._request('/Core/Test/1', 'DELETE')
        assert r.read()
        assert r.getcode() == 200, r.getcode()

    def test_get_department(self):
        from kayako.objects import Department
        d = self.api.get(Department, 1)
        self.assertEqual(d.id, 1)

    def test_create_department(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import Department
        d = self.api.create(Department)
        self.assertEqual(d.id, UnsetParameter)

    def test_creat_with_kwargs(self):
        from kayako.objects import Department
        d = self.api.create(Department, title='test_dept')
        assert d.title == 'test_dept'

    def test_creat_with_bad_kwargs(self):
        from kayako.objects import Department
        self.assertRaises(TypeError, self.api.create, Department, bad_kwarg='bad_kwarg')







# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 9, 2011

@author: evan
'''

import unittest

class TestParameterObject(unittest.TestCase):

    def test_parameters(self):
        from kayako.core.lib import ParameterObject, UnsetParameter

        class obj(ParameterObject):
            __parameters__ = ['prop1', 'prop2', 'resp1']
            __request_parameters__ = ['prop1', 'prop2']
            __response_parameters__ = ['resp1']

        o = obj(prop1='new')

        assert hasattr(o, 'prop1')
        assert hasattr(o, 'prop2')
        assert o.prop1 == 'new'
        assert o.prop2 == UnsetParameter
        assert not o.prop2

        o.resp1 = 'test'

        params = o._parameters_from_list(o.__request_parameters__)
        assert 'prop1' in params
        assert 'prop2' not in params
        assert 'resp1' not in params
        respparams = o._parameters_from_list(o.__response_parameters__)
        assert 'prop1' not in respparams
        assert 'prop2' not in respparams
        assert respparams['resp1'] == 'test'
        assert len(params) == 1

        assert len(o.parameters) == 2

    def test_multiple_objects_can_have_different_values(self):
        from kayako.core.lib import ParameterObject

        class obj(ParameterObject):
            __parameters__ = ['prop1', 'prop2']
            __request_parameters__ = ['prop1', 'prop2']

        obj1 = obj(prop1='old')
        obj2 = obj(prop1='new')

        assert obj1.prop1 != obj2.prop1

    def test_invalid_parameter(self):
        from kayako.core.lib import ParameterObject

        class obj(ParameterObject):
            __parameters__ = ['prop1', 'prop2']
            __request_parameters__ = ['prop1', 'prop2']

        self.assertRaises(TypeError, obj, prop3='fake')



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 5, 2011

@author: evan
'''

from lxml import etree

from kayako.core.object import KayakoObject

__all__ = [
    'Department',
]

class Department(KayakoObject):
    '''
    Kayako Department API Object.
    
    title                 The title of the department.
    module                The module the department should be associated with ('tickets' or 'livechat').
    type                  The accessibility level of the department ('public' or 'private').
    displayorder          A positive integer that the helpdesk will use to sort departments when displaying them (ascending).
    parentdepartmentid    A positive integer of the parent department for this department.
    uservisibilitycustom  1 or 0 boolean that controls whether or not to restrict visibility of this department to particular user groups (see usergroupid[]).
    usergroupid[]         A list of usergroup id's identifying the user groups to be assigned to this department. 
    '''

    controller = '/Base/Department'

    __parameters__ = ['id', 'title', 'type', 'module', 'displayorder', 'parentdepartmentid', 'uservisibilitycustom', 'usergroupid']

    __required_add_parameters__ = ['title', 'module', 'type']
    __add_parameters__ = ['title', 'module', 'type', 'displayorder', 'parentdepartmentid', 'uservisibilitycustom', 'usergroupid']

    __required_save_parameters__ = ['title']
    __save_parameters__ = ['title', 'type', 'displayorder', 'parentdepartmentid', 'uservisibilitycustom', 'usergroupid']


    @classmethod
    def _parse_department(cls, department_tree):
        usergroups = []
        usergroups_node = department_tree.find('usergroups')
        if usergroups_node is not None:
            for id_node in usergroups_node.findall('id'):
                id = cls._get_int(id_node)
                usergroups.append(id)

        params = dict(
            id=cls._get_int(department_tree.find('id')),
            title=cls._get_string(department_tree.find('title')),
            type=cls._get_string(department_tree.find('type')),
            module=cls._get_string(department_tree.find('module')),
            displayorder=cls._get_int(department_tree.find('displayorder')),
            parentdepartmentid=cls._get_int(department_tree.find('parentdepartmentid'), required=False),
            uservisibilitycustom=cls._get_boolean(department_tree.find('uservisibilitycustom')),
            usergroupid=usergroups,
        )
        return params

    @classmethod
    def get_all(cls, api):
        response = api._request(cls.controller, 'GET')
        tree = etree.parse(response)
        return [Department(api, **cls._parse_department(department_tree)) for department_tree in tree.findall('department')]

    @classmethod
    def get(cls, api, id):
        response = api._request('%s/%s/' % (cls.controller, id), 'GET')
        tree = etree.parse(response)
        node = tree.find('department')
        if node is None:
            return None
        params = cls._parse_department(node)
        return Department(api, **params)

    def add(self):
        response = self._add(self.controller)
        tree = etree.parse(response)
        print etree.tostring(tree)
        self.id = int(tree.find('department').find('id').text)

    def save(self):
        self._save('%s/%s/' % (self.controller, self.id))

    def delete(self):
        self._delete('%s/%s/' % (self.controller, self.id))

    def __str__(self):
        return '<Department (%s): %s/%s>' % (self.id, self.title, self.module)









# -*- coding: utf-8 -*-
'''
Created on May 10, 2011

@author: evan
'''

from kayako.tests import KayakoAPITest

class TestTicketAttachment(KayakoAPITest):

    def test_get_nonexistant(self):
        from kayako.objects import Department, Ticket, TicketAttachment
        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        obj = api.get(TicketAttachment, ticket.id, 'abc123')

        ticket.delete()

        assert obj is None

    def test_add_get(self):
        from kayako.objects import Department, Ticket, TicketAttachment, TicketPost

        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        ticket_post = api.create(TicketPost)
        ticket_post.ticketid = ticket.id
        ticket_post.subject = 'test_post'
        ticket_post.contents = 'testing a post'
        ticket_post.userid = 1
        ticket_post.add()

        ticket_attachment = api.create(TicketAttachment)
        ticket_attachment.ticketid = ticket.id
        ticket_attachment.ticketpostid = ticket_post.id
        ticket_attachment.filename = 'test_file.txt'
        ticket_attachment.set_contents('this is just a test')
        ticket_attachment.add()

        obj2 = api.get(TicketAttachment, ticket.id, ticket_attachment.id)

        ticket_attachment.delete()
        ticket_post.delete()
        ticket.delete()
        assert obj2 is not None


    def test_get_all(self):
        from kayako.objects import Department, Ticket, TicketAttachment, TicketPost

        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        ticket_post = api.create(TicketPost)
        ticket_post.ticketid = ticket.id
        ticket_post.subject = 'test_post'
        ticket_post.contents = 'testing a post'
        ticket_post.userid = 1
        ticket_post.add()

        ticket_attachment = api.create(TicketAttachment)
        ticket_attachment.ticketid = ticket.id
        ticket_attachment.ticketpostid = ticket_post.id
        ticket_attachment.filename = 'test_file.txt'
        ticket_attachment.set_contents('this is just a test')
        ticket_attachment.add()

        result = self.api.get_all(TicketAttachment, ticket.id)
        assert isinstance(result, list)
        assert result[0].get_contents() is None # Contents don't load w/ get_all

        ticket_attachment.delete()
        ticket_post.delete()
        ticket.delete()

    def test_get_ticket_attachment(self):
        from kayako.objects import Department, Ticket, TicketAttachment, TicketPost

        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        ticket_post = api.create(TicketPost)
        ticket_post.ticketid = ticket.id
        ticket_post.subject = 'test_post'
        ticket_post.contents = 'testing a post'
        ticket_post.userid = 1
        ticket_post.add()

        ticket_attachment = api.create(TicketAttachment)
        ticket_attachment.ticketid = ticket.id
        ticket_attachment.ticketpostid = ticket_post.id
        ticket_attachment.filename = 'test_file.txt'
        ticket_attachment.set_contents('this is just a test')
        ticket_attachment.add()

        result = self.api.get(TicketAttachment, ticket.id, ticket_attachment.id)
        assert result.id == ticket_attachment.id
        assert result.get_contents() == 'this is just a test'

        assert 'TicketAttachment ' in str(result)

        ticket_attachment.delete()
        ticket_post.delete()
        ticket.delete()

    def test_add_missing_ticketid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketAttachment

        ticket_attachment = self.api.create(TicketAttachment)

        ticket_attachment.ticketpostid = 1
        ticket_attachment.filename = 'test_file.txt'
        ticket_attachment.set_contents('this is just a test')

        self.assertRaises(KayakoRequestError, ticket_attachment.add)

    def test_add_missing_ticketpostid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketAttachment

        ticket_attachment = self.api.create(TicketAttachment)

        ticket_attachment.ticketid = 1
        ticket_attachment.filename = 'test_file.txt'
        ticket_attachment.set_contents('this is just a test')

        self.assertRaises(KayakoRequestError, ticket_attachment.add)

    def test_add_missing_filename(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketAttachment

        ticket_attachment = self.api.create(TicketAttachment)

        ticket_attachment.ticketid = 1
        ticket_attachment.ticketpostid = 1
        ticket_attachment.set_contents('this is just a test')

        self.assertRaises(KayakoRequestError, ticket_attachment.add)

    def test_add_missing_contents(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketAttachment

        ticket_attachment = self.api.create(TicketAttachment)

        ticket_attachment.ticketid = 1
        ticket_attachment.ticketpostid = 1
        ticket_attachment.filename = 'test_file.txt'

        self.assertRaises(KayakoRequestError, ticket_attachment.add)


    def test_add_delete(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import Department, Ticket, TicketAttachment, TicketPost

        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        ticket_post = api.create(TicketPost)
        ticket_post.ticketid = ticket.id
        ticket_post.subject = 'DELETE_ME'
        ticket_post.contents = 'testing a post'
        ticket_post.userid = 1
        ticket_post.add()

        ticket_attachment = api.create(TicketAttachment)
        ticket_attachment.ticketid = ticket.id
        ticket_attachment.ticketpostid = ticket_post.id
        ticket_attachment.filename = 'test_file.txt'
        ticket_attachment.set_contents('this is just a test')
        ticket_attachment.add()
        assert ticket_attachment.id is not UnsetParameter
        ticket_attachment.delete()
        ticket_post.delete()
        ticket.delete()

    def test_delete_unadded(self):
        from kayako.core.lib import UnsetParameter
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketAttachment
        ticket_attachment = self.api.create(TicketAttachment)
        ticket_attachment.id = UnsetParameter
        ticket_attachment.ticketid = 1
        self.assertRaises(KayakoRequestError, ticket_attachment.delete)
        ticket_attachment.id = 1
        ticket_attachment.ticketid = UnsetParameter
        self.assertRaises(KayakoRequestError, ticket_attachment.delete)



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 5, 2011

@author: evan
'''

import unittest

# TODO: Local testing

class KayakoAPITest(unittest.TestCase):

    API_URL = ''
    API_KEY = ''
    SECRET_KEY = ''

    @property
    def api(self):
        from kayako.api import KayakoAPI
        return KayakoAPI(self.API_URL, self.API_KEY, self.SECRET_KEY)



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 17, 2011

@author: evan
'''

import unittest

class TestException(unittest.TestCase):

    def test_kayako_error_no_read(self):
        from kayako.exception import KayakoError
        error = KayakoError()
        assert error.read is None

    def test_kayako_error_read_function(self):
        from StringIO import StringIO
        from kayako.exception import KayakoError
        error = KayakoError(StringIO('abc'))
        assert error.read == 'abc'

    def test_kayako_error_read_attribute(self):
        from kayako.exception import KayakoError
        class Read(object):
            read = 'abc123'
        error = KayakoError(Read())
        assert error.read == 'abc123'





# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
from kayako.api import KayakoAPI
from kayako.core.lib import UnsetParameter, FOREVER
from kayako.objects import *

__NAME__ = 'kayako'
__VERSION__ = '1.0.03'
__AUTHOR__ = 'Evan Leis'
__EMAIL__ = 'engineergod@yahoo.com'



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 5, 2011

@author: evan
'''

from datetime import datetime

__all__ = [
    'UnsetParameter',
    'FOREVER',
    'ParameterObject',
    'NodeParser',
]

class _unsetparameter(object):

    def __nonzero__(self):
        return False

    def __call__(self):
        return self

    def __repr__(self):
        return 'UnsetParameter()'

    def __str__(self):
        return '??'

class _forever(object):

    def __int__(self):
        return 0

    def __repr__(self):
        return '0'

    def __str__(self):
        return '<Forever>'

UnsetParameter = _unsetparameter()
FOREVER = _forever()

class ParameterObject(object):
    '''
    An object used to build a dictionary around different parameter types.
    '''

    __parameters__ = []
    ''' Parameters that this ParameterObject can have. '''

    def __init__(self, **parameters):
        '''
        Creates this parameter object setting parameter values as given by
        keyword arguments.
        '''
        unset_parameters = self.__parameters__[:]
        for parameter, value in parameters.iteritems():
            if parameter not in self.__parameters__:
                raise TypeError("'%s' is an invalid keyword argument for %s" % (parameter, self.__class__.__name__))
            else:
                setattr(self, parameter, value)
                unset_parameters.remove(parameter)
        for unset_parameter in unset_parameters:
            setattr(self, unset_parameter, UnsetParameter)

    @property
    def parameters(self):
        return self._parameters_from_list(self.__parameters__)

    def _parameters_from_list(self, list):
        '''
        Returns parameters based on a list.
        '''
        params = {}
        for parameter in list:
            attribute = getattr(self, parameter)
            if attribute is not UnsetParameter:
                params[parameter] = attribute
        return params

    def __str__(self):
        return '<ParamterObject at %s>' % (hex(id(self)))

class NodeParser(object):
    ''' Methods to parse text data from an lxml etree object. '''

    @staticmethod
    def _parse_int(data, required=True, strict=True):
        ''' Simply parses data as an int.
        If its required, invalid data will raise a ValueError or TypeError.
        If its not required, but is strict and there is data, invalid data will
        raise a ValueError.
        If it is not required and is not strict, invalid data returns None.
        '''
        if required:
            return int(data)
        else:
            if data:
                try:
                    return int(data)
                except ValueError:
                    if strict:
                        raise

    @staticmethod
    def _parse_date(data, required=True, strict=True):
        '''
        Return an integer, or FOREVER. See _parse_int for information on 
        required and strict.
        '''
        value = NodeParser._parse_int(data, required=required, strict=strict)
        if value is None:
            return None
        elif value == 0:
            return FOREVER
        else:
            return value

    @staticmethod
    def _get_int(node, required=True, strict=True):
        '''
        Pulls out an integer from the etree on the given node.
        If it is required, it will assume everything is present, otherwise the
        return value could be None.
        If strict is True, it will parse any available text as an integer, 
        raising ValueError if it does not parse.
        Otherwise, unparsable text is ignored.  Required implies Strict.
        '''
        if required:
            return int(node.text)
        else:
            if node is not None and node.text:
                try:
                    return int(node.text)
                except ValueError:
                    if strict:
                        raise

    @staticmethod
    def _get_string(node):
        '''
        Pulls out the text of a given node.  Returns None if missing.
        '''
        if node is not None:
            return node.text

    @staticmethod
    def _get_boolean(node, required=True, strict=True):
        '''
        Returns the boolean value of an integer node.  See _get_int for details
        about required and strict.
        '''
        value = NodeParser._get_int(node, required=required, strict=strict)
        if value is None:
            return None
        else:
            if not strict:
                return bool(value)
            else:
                if value == 0:
                    return False
                elif value == 1:
                    return True
                else:
                    raise ValueError('Value for node not 1 or 0')

    @staticmethod
    def _get_date(node, required=True, strict=True):
        '''
        Return an integer, or FOREVER.  See _get_int for details about required
        and strict.
        '''
        value = NodeParser._get_int(node, required=required, strict=strict)
        if value is None:
            return None
        elif value == 0:
            return FOREVER
        else:
            return datetime.fromtimestamp(value)

    def __str__(self):
        return '<NodeParser at %s>' % (hex(id(self)))



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 10, 2011

@author: evan
'''

from kayako.tests import KayakoAPITest

class TestTicketNote(KayakoAPITest):

#    def test_get_nonexistant(self):
#        from kayako.objects import Department, Ticket, TicketNote
#
#        api = self.api
#
#        depts = api.get_all(Department)
#        for dept in depts:
#            if dept.module == 'tickets':
#                break
#
#        ticket = api.create(Ticket)
#        ticket.subject = 'DELETE_ME'
#        ticket.fullname = 'Unit Test'
#        ticket.email = 'test@example.com'
#        ticket.contents = 'test'
#        ticket.departmentid = dept.id
#        ticket.ticketstatusid = 1
#        ticket.ticketpriorityid = 1
#        ticket.tickettypeid = 1
#        ticket.userid = 1
#        ticket.ownerstaffid = 1
#        ticket.type = 'default'
#        ticket.add()
#
#        obj = api.get(TicketNote, ticket.id, 'abc123')
#
#        ticket.delete()
#
#        assert obj is None
#
#    def test_add_get_bare_staffid(self):
#        from kayako.objects import Department, Ticket, TicketNote
#
#        api = self.api
#
#        depts = api.get_all(Department)
#        for dept in depts:
#            if dept.module == 'tickets':
#                break
#
#        ticket = api.create(Ticket)
#        ticket.subject = 'DELETE_ME'
#        ticket.fullname = 'Unit Test'
#        ticket.email = 'test@example.com'
#        ticket.contents = 'test'
#        ticket.departmentid = dept.id
#        ticket.ticketstatusid = 1
#        ticket.ticketpriorityid = 1
#        ticket.tickettypeid = 1
#        ticket.userid = 1
#        ticket.ownerstaffid = 1
#        ticket.type = 'default'
#        ticket.add()
#
#        ticket_note = api.create(TicketNote)
#        ticket_note.ticketid = ticket.id
#        ticket_note.contents = 'testing a post'
#        ticket_note.staffid = 1
#        ticket_note.add()
#
#        obj2 = api.get(TicketNote, ticket.id, ticket_note.id)
#        assert obj2 is not None
#        ticket.delete()
#
#    def test_add_get_bare_fullname(self):
#        from kayako.objects import Department, Ticket, TicketNote
#
#        api = self.api
#
#        depts = api.get_all(Department)
#        for dept in depts:
#            if dept.module == 'tickets':
#                break
#
#        ticket = api.create(Ticket)
#        ticket.subject = 'DELETE_ME'
#        ticket.fullname = 'Unit Test'
#        ticket.email = 'test@example.com'
#        ticket.contents = 'test'
#        ticket.departmentid = dept.id
#        ticket.ticketstatusid = 1
#        ticket.ticketpriorityid = 1
#        ticket.tickettypeid = 1
#        ticket.userid = 1
#        ticket.ownerstaffid = 1
#        ticket.type = 'default'
#        ticket.add()
#
#        ticket_note = api.create(TicketNote)
#        ticket_note.ticketid = ticket.id
#        ticket_note.contents = 'testing a post'
#        ticket_note.fullname = 'testing'
#        ticket_note.add()
#
#        obj2 = api.get(TicketNote, ticket.id, ticket_note.id)
#        assert obj2 is not None
#        ticket.delete()
#
#    def test_add_get_full_staffid(self):
#        from kayako.objects import Department, Ticket, TicketNote
#
#        api = self.api
#
#        depts = api.get_all(Department)
#        for dept in depts:
#            if dept.module == 'tickets':
#                break
#
#        ticket = api.create(Ticket)
#        ticket.subject = 'DELETE_ME'
#        ticket.fullname = 'Unit Test'
#        ticket.email = 'test@example.com'
#        ticket.contents = 'test'
#        ticket.departmentid = dept.id
#        ticket.ticketstatusid = 1
#        ticket.ticketpriorityid = 1
#        ticket.tickettypeid = 1
#        ticket.userid = 1
#        ticket.ownerstaffid = 1
#        ticket.type = 'default'
#        ticket.add()
#
#        ticket_note = api.create(TicketNote)
#        ticket_note.ticketid = ticket.id
#        ticket_note.subject = 'test_post'
#        ticket_note.contents = 'testing a post'
#        ticket_note.staffid = 1
#        ticket_note.notecolor = 1
#        ticket_note.forstaff_id = 1
#        ticket_note.add()
#
#        obj2 = api.get(TicketNote, ticket.id, ticket_note.id)
#        assert obj2 is not None
#        ticket.delete()
#
#    def test_add_get_full_fullname(self):
#        from kayako.objects import Department, Ticket, TicketNote
#
#        api = self.api
#
#        depts = api.get_all(Department)
#        for dept in depts:
#            if dept.module == 'tickets':
#                break
#
#        ticket = api.create(Ticket)
#        ticket.subject = 'DELETE_ME'
#        ticket.fullname = 'Unit Test'
#        ticket.email = 'test@example.com'
#        ticket.contents = 'test'
#        ticket.departmentid = dept.id
#        ticket.ticketstatusid = 1
#        ticket.ticketpriorityid = 1
#        ticket.tickettypeid = 1
#        ticket.userid = 1
#        ticket.ownerstaffid = 1
#        ticket.type = 'default'
#        ticket.add()
#
#        ticket_note = api.create(TicketNote)
#        ticket_note.ticketid = ticket.id
#        ticket_note.subject = 'test_post'
#        ticket_note.contents = 'testing a post'
#        ticket_note.fullname = 'testing'
#        ticket_note.notecolor = 1
#        ticket_note.forstaff_id = 1
#        ticket_note.add()
#
#        obj2 = api.get(TicketNote, ticket.id, ticket_note.id)
#        assert obj2 is not None
#        ticket.delete()


    def test_get_all(self):
        from kayako.objects import Department, Ticket, TicketNote

        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        ticket_note = api.create(TicketNote)
        ticket_note.ticketid = ticket.id
        ticket_note.subject = 'test_post'
        ticket_note.contents = 'testing a post'
        ticket_note.staffid = 1
        ticket_note.add()

        result = self.api.get_all(TicketNote, ticket.id)
        assert isinstance(result, list)
        assert result

        ticket = api.get(Ticket, ticket.id)
        assert ticket.notes[0].ticketid == ticket.id

        assert 'TicketNote ' in str(ticket.notes[0])

        ticket.delete()


    def test_add_missing_ticketid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketNote

        ticket_note = self.api.create(TicketNote)

        ticket_note.ticketid = 1
        ticket_note.staffid = 1

        self.assertRaises(KayakoRequestError, ticket_note.add)

    def test_add_missing_contents(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketNote

        ticket_note = self.api.create(TicketNote)

        ticket_note.contents = 'this is just a test'
        ticket_note.staffid = 1

        self.assertRaises(KayakoRequestError, ticket_note.add)

    def test_add_missing_staffid_and_fullname(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketNote

        ticket_note = self.api.create(TicketNote)

        ticket_note.contents = 'this is just a test'

        self.assertRaises(KayakoRequestError, ticket_note.add)

    def test_add_both_staffid_and_fullname(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketNote

        ticket_note = self.api.create(TicketNote)

        ticket_note.ticketid = 1
        ticket_note.contents = 'this is just a test'
        ticket_note.staffid = 1
        ticket_note.fullname = 'test name'

        self.assertRaises(KayakoRequestError, ticket_note.add)



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 9, 2011

@author: evan
'''
from kayako.tests import KayakoAPITest

class TestTicket(KayakoAPITest):

    def test_get_nonexistant(self):
        from kayako.objects import Ticket
        obj = self.api.get(Ticket, 123123123)
        assert obj is None

    def test_add_get_bare_userid(self):
        from kayako.objects import Department, Ticket

        depts = self.api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        obj = self.api.create(Ticket, subject='DELETEME', fullname='DELETE ME', email='deleteme@example.com', contents='DELETE ME', departmentid=dept.id, ticketstatusid=1, ticketpriorityid=1, tickettypeid=1, userid=1)
        obj.add()
        obj2 = self.api.get(Ticket, obj.id)
        obj.delete()
        assert obj2 is not None

    def test_add_get_bare_staffid(self):
        from kayako.objects import Department, Ticket

        depts = self.api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        obj = self.api.create(Ticket, subject='DELETEME', fullname='DELETE ME', email='deleteme@example.com', contents='DELETE ME', departmentid=dept.id, ticketstatusid=1, ticketpriorityid=1, tickettypeid=1, staffid=1)
        obj.add()
        obj2 = self.api.get(Ticket, obj.id)
        obj.delete()
        assert obj2 is not None

    def test_add_get_full_userid(self):
        from kayako.objects import Department, Ticket

        depts = self.api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        obj = self.api.create(Ticket, subject='DELETEME', fullname='DELETE ME', email='deleteme@example.com', contents='DELETE ME', departmentid=dept.id, ticketstatusid=1, ticketpriorityid=1, tickettypeid=1,
                              userid=1, ownerstaffid=1, type='default')
        obj.add()
        obj2 = self.api.get(Ticket, obj.id)
        obj.delete()
        assert obj2 is not None

    def test_add_get_full_staffid(self):
        from kayako.objects import Department, Ticket

        depts = self.api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        obj = self.api.create(Ticket, subject='DELETEME', fullname='DELETE ME', email='deleteme@example.com', contents='DELETE ME', departmentid=dept.id, ticketstatusid=1, ticketpriorityid=1, tickettypeid=1,
                              staffid=1, ownerstaffid=1, type='default')
        obj.add()
        obj2 = self.api.get(Ticket, obj.id)
        obj.delete()
        assert obj2 is not None

    def test_get_all(self):
        from kayako.objects import Department, Ticket

        depts = self.api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        result = self.api.get_all(Ticket, dept.id)
        assert isinstance(result, list)

    def test_get_all_with_kwargs(self):
        from kayako.objects import Department, Ticket

        depts = self.api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = self.api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        result = self.api.get_all(Ticket, dept.id, ticketstatusid=[1], ownerstaffid=[1], userid=[1])
        assert isinstance(result, list)

        ticket.delete()

    def test_get_ticket(self):
        from kayako.objects import Department, Ticket

        depts = self.api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = self.api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        get_ticket = self.api.get(Ticket, ticket.id)
        self.assertEqual(get_ticket.id, ticket.id)


        assert 'Ticket ' in str(get_ticket)

        ticket.delete()

    def test_add_missing_subject(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket

        ticket = self.api.create(Ticket)

        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = 1
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1

        ticket.userid = 1

        ticket.ownerstaffid = 1
        ticket.type = 'default'

        self.assertRaises(KayakoRequestError, ticket.add)

    def test_add_missing_fullname(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket

        ticket = self.api.create(Ticket)

        ticket.subject = 'TEST TICKET'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = 1
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1

        ticket.userid = 1

        ticket.ownerstaffid = 1
        ticket.type = 'default'

        self.assertRaises(KayakoRequestError, ticket.add)

    def test_add_missing_email(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket

        ticket = self.api.create(Ticket)

        ticket.subject = 'TEST TICKET'
        ticket.fullname = 'Unit Test'
        ticket.contents = 'test'
        ticket.departmentid = 1
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1

        ticket.userid = 1

        ticket.ownerstaffid = 1
        ticket.type = 'default'

        self.assertRaises(KayakoRequestError, ticket.add)

    def test_add_missing_contents(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket

        ticket = self.api.create(Ticket)

        ticket.subject = 'TEST TICKET'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.departmentid = 1
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1

        ticket.userid = 1

        ticket.ownerstaffid = 1
        ticket.type = 'default'

        self.assertRaises(KayakoRequestError, ticket.add)

    def test_add_missing_departmentid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket

        ticket = self.api.create(Ticket)

        ticket.subject = 'TEST TICKET'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1

        ticket.userid = 1

        ticket.ownerstaffid = 1
        ticket.type = 'default'

        self.assertRaises(KayakoRequestError, ticket.add)

    def test_add_missing_ticketstatusid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket

        ticket = self.api.create(Ticket)

        ticket.subject = 'TEST TICKET'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1

        ticket.userid = 1

        ticket.ownerstaffid = 1
        ticket.type = 'default'

        self.assertRaises(KayakoRequestError, ticket.add)

    def test_add_missing_ticketpriorityid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket

        ticket = self.api.create(Ticket)

        ticket.subject = 'TEST TICKET'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = 1
        ticket.ticketstatusid = 1
        ticket.tickettypeid = 1

        ticket.userid = 1

        ticket.ownerstaffid = 1
        ticket.type = 'default'

        self.assertRaises(KayakoRequestError, ticket.add)

    def test_add_missing_tickettypeid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket

        ticket = self.api.create(Ticket)

        ticket.subject = 'TEST TICKET'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = 1
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1

        ticket.userid = 1

        ticket.ownerstaffid = 1
        ticket.type = 'default'

        self.assertRaises(KayakoRequestError, ticket.add)

    def test_add_missing_staffid_and_userid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket

        ticket = self.api.create(Ticket)

        ticket.subject = 'TEST TICKET'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = 1
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1

        ticket.ownerstaffid = 1
        ticket.type = 'default'

        self.assertRaises(KayakoRequestError, ticket.add)

    def test_add_with_both_staffid_and_userid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket

        ticket = self.api.create(Ticket)

        ticket.subject = 'TEST TICKET'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = 1
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1

        ticket.userid = 1
        ticket.staffid = 1

        ticket.ownerstaffid = 1
        ticket.type = 'default'

        self.assertRaises(KayakoRequestError, ticket.add)


    def test_add_save_delete(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import Department, Ticket

        depts = self.api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = self.api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()
        assert ticket.id is not UnsetParameter
        ticket.subject = 'DELETE_ME2'
        ticket.save()
        ticket.delete()

        found_error = False
        all = self.api.get_all(Ticket, 1, ticketstatusid=1, ownerstaffid=1, userid=1)
        for obj in all:
            if obj.subject == 'DELETE_ME' or obj.subject == 'DELETE_ME2':
                obj.delete()
                found_error = True
        if found_error:
            assert False, 'Found an error, Tickets did not delete correctly.'

    def test_delete_unadded(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Ticket
        ticket = self.api.create(Ticket)
        self.assertRaises(KayakoRequestError, ticket.delete)




# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 9, 2011

@author: evan
'''
from kayako.tests import KayakoAPITest

class TestDepartment(KayakoAPITest):

    def test_get_nonexistant(self):
        from kayako.objects import Department
        d = self.api.get(Department, '123123')
        assert d is None

    def test_add_get_bare(self):
        from kayako.objects import Department
        d = self.api.create(Department, title='test', module='tickets', type='private')
        d.add()
        obj2 = self.api.get(Department, d.id)
        d.delete()
        assert obj2 is not None

    def test_add_get_full(self):
        from kayako.objects import Department
        parent = self.api.first(Department, parentdepartmentid=0)
        d = self.api.create(Department, title='test', module='tickets', type='private',
                            displayorder=15, parentdepartmentid=parent.id, uservisibilitycustom=0, usergroupid=[1, 2])
        d.add()
        obj2 = self.api.get(Department, d.id)
        d.delete()
        assert obj2 is not None

    def test_get_department(self):
        from kayako.objects import Department
        d = self.api.get(Department, 1)
        assert 'Department' in str(d)
        self.assertEqual(d.id, 1)

    def test_add_department_missing_title(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Department
        department = self.api.create(Department)

        department.module = 'tickets'
        department.type = 'public'

        self.assertRaises(KayakoRequestError, department.add)

    def test_add_department_missing_module(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Department
        department = self.api.create(Department)

        department.title = 'test'
        department.type = 'public'

        self.assertRaises(KayakoRequestError, department.add)

    def test_add_department_missing_type(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Department
        department = self.api.create(Department)

        department.title = 'test'
        department.module = 'tickets'

        self.assertRaises(KayakoRequestError, department.add)


    def test_get_all(self):
        from kayako.objects import Department
        result = self.api.get_all(Department)
        assert len(result)

    def test_add_save_delete(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import Department

        department = self.api.create(Department)
        department.title = 'DELETE_ME'
        department.module = 'tickets'
        department.type = 'private'
        department.add()
        assert department.id is not UnsetParameter
        department.title = 'DELETE_ME2'
        department.save()
        department.delete()

        found_error = False
        all_depts = self.api.get_all(Department)
        for dept in all_depts:
            if dept.title == 'DELETE_ME' or dept.title == 'DELETE_ME2':
                dept.delete()
                found_error = True
        if found_error:
            assert False, 'Found an error, Departments did not delete correctly.'

    def test_delete_unadded(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import Department
        department = self.api.create(Department)
        self.assertRaises(KayakoRequestError, department.delete)





# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 5, 2011

@author: evan
'''

class KayakoError(StandardError):

    @property
    def read(self):
        if self.args:
            for arg in self.args:
                if hasattr(arg, 'read'):
                    if callable(arg.read):
                        return arg.read()
                    else:
                        return arg.read

class KayakoInitializationError(KayakoError):
    pass

# REQUEST ERRORS

class KayakoRequestError(KayakoError):
    pass

class KayakoMethodNotImplementedError(KayakoRequestError):
    '''
    An exception for when an HTTP request method is not implemented for an
    object.
    '''
    pass

# RESPONSE ERROR

class KayakoResponseError(KayakoError):
    pass



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------

from setuptools import setup, find_packages

with open('kayako/README.txt') as readme:
    long_description = readme.read()

setup(
    name='kayako',
    version='1.1.01',
    description="Python API Wrapper for Kayako 4.01.204",
    long_description=long_description,
    author='Evan Leis',
    author_email='engineergod@yahoo.com',
    url='',
    install_requires=[
        'lxml',
    ],
    setup_requires=[],
    packages=find_packages(exclude=[]),
    include_package_data=True,
    test_suite='kayako.tests',
    package_data={},
    zip_safe=True,
    entry_points="""
    """,
    license="Lesser GNU General Public License (LGPL)",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Development Status :: 4 - Beta",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 9, 2011

@author: evan
'''
from kayako.tests import KayakoAPITest

class TestUser(KayakoAPITest):

    def test_get_nonexistant(self):
        from kayako.objects import User
        obj = self.api.get(User, 'abc123')
        assert obj is None

    def test_add_get_bare(self):
        from kayako.objects import User
        user = self.api.create(User, fullname='DELETE ME', usergroupid=2, password='deleteme', email='deleteme@example.com')
        user.add()
        user2 = self.api.get(User, user.id)
        user.delete()
        assert user2 is not None

    def test_add_get_full(self):
        from datetime import datetime
        from kayako.objects import User
        user = self.api.create(User, fullname='DELETE ME', usergroupid=2, password='deleteme', email='deleteme@example.com',
                               userorganizationid=1, salutation='Mr.', designation='CEO', phone='123-456-7890', isenabled=False, userrole='user', timezone='MST', enabledst=True, slaplanid=1, slaplanexpiry=datetime.now(), sendwelcomeemail=False)
        user.add()
        user2 = self.api.get(User, user.id)
        user.delete()
        assert user2 is not None

    def test_get_all(self):
        from kayako.objects import User
        result = self.api.get_all(User)
        assert len(result)

    def test_get_all_with_kwargs(self):
        from kayako.objects import User
        result = self.api.get_all(User, marker=200, maxitems=50)
        assert isinstance(result, list)

    def test_get_user(self):
        from kayako.objects import User
        user = self.api.get(User, 1)

        assert 'User ' in str(user)

        self.assertEqual(user.id, 1)

    def test_add_user_missing_fullname(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import User
        user = self.api.create(User)

        user.usergroupid = -1
        user.password = 'test_password'
        user.email = 'test_email@example.com'
        user.dateline = 4234
        user.phone = '56465464'

        self.assertRaises(KayakoRequestError, user.add)

    def test_add_user_missing_usergroupid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import User
        user = self.api.create(User)

        user.fullname = 'test_user'
        user.password = 'test_password'
        user.email = 'test_email@example.com'
        user.dateline = 4234
        user.phone = '56465464'

        self.assertRaises(KayakoRequestError, user.add)

    def test_add_user_missing_password(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import User
        user = self.api.create(User)

        user.fullname = 'test_user'
        user.usergroupid = -1
        user.email = 'test_email@example.com'
        user.dateline = 4234
        user.phone = '56465464'

        self.assertRaises(KayakoRequestError, user.add)

    def test_add_user_missing_email(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import User
        user = self.api.create(User)

        user.fullname = 'test_user'
        user.usergroupid = -1
        user.password = 'test_password'
        user.dateline = 4234
        user.phone = '56465464'

        self.assertRaises(KayakoRequestError, user.add)

    def test_add_save_delete(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import User

        user = self.api.create(User)
        user.fullname = 'DELETE_ME'
        user.usergroupid = 2
        user.password = 'test_password'
        user.email = 'test@example.com'
        user.add()
        assert user.id is not UnsetParameter
        user.fullname = 'DELETE_ME2'
        user.save()
        user.delete()

        found_error = False
        all = self.api.get_all(User)
        for obj in all:
            if obj.fullname == 'DELETE_ME' or obj.fullname == 'DELETE_ME2':
                obj.delete()
                found_error = True
        if found_error:
            assert False, 'Found an error, Users did not delete correctly.'

    def test_delete_unadded(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import User
        user = self.api.create(User)
        self.assertRaises(KayakoRequestError, user.delete)

class TestUserGroup(KayakoAPITest):

    def test_get_nonexistant(self):
        from kayako.objects import UserGroup
        obj = self.api.get(UserGroup, '12345')
        print obj
        assert obj is None

    def test_add_get(self):
        from kayako.objects import UserGroup
        obj = self.api.create(UserGroup, title='DELETE ME', grouptype='registered')
        obj.add()
        obj2 = self.api.get(UserGroup, obj.id)
        obj.delete()
        assert obj2 is not None

    def test_get_all(self):
        from kayako.objects import UserGroup
        result = self.api.get_all(UserGroup)
        assert len(result)

    def test_get(self):
        from kayako.objects import UserGroup
        usergroup = self.api.get(UserGroup, 1)
        self.assertEqual(usergroup.id, 1)

    def test_add_missing_title(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import UserGroup
        usergroup = self.api.create(UserGroup)

        usergroup.grouptype = 'registered'

        self.assertRaises(KayakoRequestError, usergroup.add)

    def test_add_missing_grouptype(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import UserGroup
        usergroup = self.api.create(UserGroup)

        usergroup.title = 'test_usergroup'

        self.assertRaises(KayakoRequestError, usergroup.add)


    def test_add_save_delete(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import UserGroup

        usergroup = self.api.create(UserGroup)
        usergroup.title = 'DELETE_ME'
        usergroup.grouptype = 'registered'
        usergroup.add()
        assert usergroup.id is not UnsetParameter
        usergroup.title = 'DELETE_ME2'
        usergroup.save()
        usergroup.delete()

        found_error = False
        all = self.api.get_all(UserGroup)
        for obj in all:
            if obj.title == 'DELETE_ME' or obj.title == 'DELETE_ME2':
                obj.delete()
                found_error = True
        if found_error:
            assert False, 'Found an error, UserGroups did not delete correctly.'

    def test_delete_unadded(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import UserGroup
        usergroup = self.api.create(UserGroup)
        self.assertRaises(KayakoRequestError, usergroup.delete)

class TestUserOrganization(KayakoAPITest):

    def test_get_nonexistant(self):
        from kayako.objects import UserOrganization
        obj = self.api.get(UserOrganization, '123123')
        assert obj is None

    def test_add_get_bare(self):
        from kayako.objects import UserOrganization
        obj = self.api.create(UserOrganization, name='DELETE ME', organizationtype='restricted')
        obj.add()
        obj2 = self.api.get(UserOrganization, obj.id)
        obj.delete()
        assert obj2 is not None

    def test_add_get_full(self):
        from datetime import datetime
        from kayako.objects import UserOrganization
        obj = self.api.create(UserOrganization, name='DELETE ME', organizationtype='restricted',
                              address='23423 E', city='Salt Lake City', state='Utah', postalcode=84121, country='USA', phone='123-456-7890', fax='098-765-4321', website='http://example.com', slaplanid=1, slaplanexpiry=datetime.now())
        obj.add()
        obj2 = self.api.get(UserOrganization, obj.id)
        obj.delete()
        assert obj2 is not None

    def test_get_all(self):
        from kayako.objects import UserOrganization
        result = self.api.get_all(UserOrganization)
        assert len(result)

    def test_get(self):
        from kayako.objects import UserOrganization
        userorg = self.api.get(UserOrganization, 1)
        self.assertEqual(userorg.id, 1)

    def test_add_missing_name(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import UserOrganization
        userorg = self.api.create(UserOrganization)

        userorg.organizationtype = 'restricted'

        self.assertRaises(KayakoRequestError, userorg.add)

    def test_add_missing_organizationtype(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import UserOrganization
        userorg = self.api.create(UserOrganization)

        userorg.name = 'test_user_org'

        self.assertRaises(KayakoRequestError, userorg.add)

    def test_add_save_delete(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import UserOrganization

        userorg = self.api.create(UserOrganization)
        userorg.name = 'DELETE_ME'
        userorg.organizationtype = 'restricted'
        userorg.add()
        assert userorg.id is not UnsetParameter
        userorg.name = 'DELETE_ME2'
        userorg.save()
        userorg.delete()

        found_error = False
        all = self.api.get_all(UserOrganization)
        for obj in all:
            if obj.name == 'DELETE_ME' or obj.name == 'DELETE_ME2':
                obj.delete()
                found_error = True
        if found_error:
            assert False, 'Found an error, UserOrganizations did not delete correctly.'

    def test_delete_unadded(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import UserOrganization
        userorg = self.api.create(UserOrganization)
        self.assertRaises(KayakoRequestError, userorg.delete)

